import React, { PureComponent } from 'react';
import ReactNative, {
    View,
    Text,
    Platform,
    requireNativeComponent,
    UIManager,
    ListView,
    SectionList,
} from 'react-native';

import PropTypes from 'prop-types';

import Refreshing from './Refreshing';
import LoadingMore from './LoadingMore';

import RecyclerView from '../recyclerview/RecyclerView';

/**
 * 下拉头部最大高度
 * @type {number}
 */
const MAX_REFRESH_HEIGHT = 50;

import PullToRefreshContainerV2 from '../pulltorefresh/PullToRefreshContainerV2';

/**
 *
 */
export default class RefreshListView extends PureComponent {

    /**
     *
     * @type {{useRefreshViewDefault: React$PropType$Primitive.<boolean>, enabledPullDown: React$PropType$Primitive.<boolean>, enabledPullUp: React$PropType$Primitive.<boolean>, onRefresh, onLoadMore}}
     */
    static propTypes = {
        enabledPullDown:PropTypes.bool,		//是否开启下拉刷新
        enabledPullUp:PropTypes.bool,		//是否开启上拉加载
        onRefresh:PropTypes.func,   //下拉刷新
        onLoadMore:PropTypes.func,   //上拉加载
    }

    /**
     *
     * @type {{enabledPullDown: boolean, enabledPullUp: boolean}}
     */
    static defaultProps = {
        enabledPullDown: false,
        enabledPullUp: false
    }

    /**
     *
     * @type {null}
     * @private
     */
    _ds = null;
    /**
     *
     * @type {Array}
     * @private
     */
    _dataList = [];
    /**
     * 是否正在加载更多
     * @type {boolean}
     */
    isLoadMore = false;

    constructor(props) {
        super(props);

        const { dataSource, dataList } = this.props;

        if(dataSource){
            this.state = {
                dataSource:dataSource,
            }
        }else{
            this._dataList.addAll(dataList);
            this._ds = new ListView.DataSource({rowHasChanged: (r1, r2) => r1 !== r2});
            this.state={
                dataSource:this._ds.cloneWithRows(this._dataList),
            }
        }
    }

    /**
     * 停止滚动
     */
    stopScroll = ()=>{
        this.refs.recyclerView && this.refs.recyclerView.stopScroll && this.refs.recyclerView.stopScroll();
    }

    /**
     *
     * @param dataSource
     */
    setDataSource = (dataSource : ListView.DataSource)=>{
        this.setState({ dataSource });
    }

    /**
     *
     * @param item
     * @param index
     */
    addItemData = (item : any, index : number)=>{
        if(this._ds){
            if(typeof index === 'string'){
                index = Number(index);
            }
            this._dataList.add(item, index);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param item
     */
    addAllItemData = (itemArray : Array) => {
        if(this._ds){
            this._dataList.addAll(itemArray);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param position
     */
    removeItemDataAtPosition = (position : number)=>{
        if(this._ds){
            if(typeof position === 'string'){
                position = Number(position);
            }
            this._dataList.remove(position);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     *
     * @param position
     */
    removeItemData = (item : any)=>{
        if(this._ds){
            this._dataList.remove(item);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 清空数据
     */
    clear = ()=>{
        if(this._ds){
            this._dataList = [];
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 清空数据
     */
    clearAndAddAll = (itemArray : Array = [])=>{
        if(this._ds){
            this._dataList = [];
            this._dataList.addAll(itemArray);
            this.setState({
                dataSource:this._ds.cloneWithRows(this._dataList)
            });
        }
    }

    /**
     * 数据总数
     * @returns {number}
     */
    getItemCount = () : number =>{
        return this.state.dataSource.getRowCount();
    }

    /**
     * 是否正在下拉刷新
     * @returns {*|boolean}
     */
    get isRefreshing(){
        return this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer._isRefreshing;
    }

    /**
     * 自动下拉刷新
     */
    beginRefreshing = () => {
        if (!this.isRefreshing && !this.isLoadMore) {
            this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer.beginRefreshing();
        }
    }

    /**
     * 结束下拉刷新
     */
    endRefreshing = () => {
        this.refs.pullToRefreshContainer && this.refs.pullToRefreshContainer.endRefreshing();
    }

    /**
     * 上拉加载结束
     * @param hasLoadAll    是否已经加载所有数据  true：底部显示没有更多了  false: 加载中...
     */
    endLoadMore = (hasLoadAll : boolean = false, ingoreLoadMore : boolean = false) => {
        if(!this.isRefreshing || ingoreLoadMore){
            this._endLoadMore(hasLoadAll);
        }
    }

    /**
     *
     * @param hasLoadAll
     * @private
     */
    _endLoadMore = (hasLoadAll : boolean = false)=>{
        if(hasLoadAll){	//没有更多
            this.refs.recyclerView && this.refs.recyclerView.onLoadMoreCompleteAndNoData();
        }else{	//加载更多结束
            this.refs.recyclerView && this.refs.recyclerView.onLoadMoreComplete();
        }
    }

    /**
     *
     * @type {e}
     * @private
     */
    _pullToRefreshContainerLayout = (e)=>{
        const { nativeEvent } = e;
        this.refs.recyclerView && this.refs.recyclerView.setNativeProps({
            style:{
                flex:0,
                height:nativeEvent.layout.height
            }
        })
        this.refs.recyclerContainerView && this.refs.recyclerContainerView.setNativeProps({
            style:{
                flex:0,
                height:nativeEvent.layout.height
            }
        })
    }

    /**
     *
     * @param e
     * @private
     */
    _onScroll = (e)=>{
        const { nativeEvent } = e;
        this._isListViewScrollTop = nativeEvent.posState && nativeEvent.posState.isTop;

        const { onScroll } = this.props;
        onScroll && onScroll(e);
    }

    /**
     * 下拉刷新开始
     * @private
     */
    _onRefresh = ()=>{
        this._endLoadMore();

        const { onRefresh } = this.props;
        onRefresh && onRefresh();
    }

    /**
     * 加载更多
     * @private
     */
    _onLoadMore = ()=>{
        const { onLoadMore } = this.props;
        onLoadMore && onLoadMore();
    }

    /**
     * 加载更多 View
     * @param txt
     * @param hasLoadAll    是否已经加载全部数据   true：显示没有更多了 false：显示正在加载中...
     * @returns {XML}
     * @private
     */
    _renderLoadMoreView = (txt : String = '盟管家正在加载', hasLoadAll : Boolean = false)=>{

        const { loadAllLineStyle } = this.props;

        return (
            <LoadingMore
                key={'loadMoreView'}
                ref={'loadMoreView'}
                style={{height: MAX_REFRESH_HEIGHT}}
                isShow={true}
                loadAllLineStyle={loadAllLineStyle}
                hasLoadAll={hasLoadAll}
                txt={txt}
            />
        );
    }

    /**
     * 是否开始下拉
     * @param evt
     * @param gestureState
     * @private
     */
    _isReadyForPullStart = (evt, gestureState)=>{
        const { dx, dy } = gestureState;
        if(dy > 0){
            if(this._isListViewScrollTop && !this.isLoadMore){
                return true;
            }
        }
    }

    /**
     * 刷新/加载状态
     * @param e
     * @private
     */
    _onRefreshState = (e)=>{
        const { nativeEvent } = e;
        const { refreshState, currState } = nativeEvent;
        if(refreshState === 'LOADMORE'){    //加载更多
            this.isLoadMore = false;
            switch (currState){
                case 'NORMAL':    //正常
                    break;
                case 'LOADMORE_LOADING':    //加载中
                    this.isLoadMore = true;
                    break;
                case 'LOADMORE_COMPLETE':    //加载完成
                    break;
                case 'LOADMORE_NOMORE':    //没有更多
                    break;
                case 'LOADMORE_FOOTER_GONE':    //底部隐藏
                    break;
            }
        }
        console.log('_onRefreshState=====> ', nativeEvent);
    }

    render() {

        const { enabledPullUp, enabledPullDown, style, contentContainerStyle, scrollEventThrottle = 200, ...otherProps } = this.props;

        const { dataSource } = this.state;

        //
        const refreshControl = (
            <PullToRefreshContainerV2
                onLayout={this._pullToRefreshContainerLayout}
                ref={'pullToRefreshContainer'}
                style={{flex:1}}
                isReadyForPullStart={this._isReadyForPullStart}
                onRefresh={this._onRefresh}
            />
        );

        const dataList = [];
        if(dataSource){
            for(let sectionIndex = 0; sectionIndex < dataSource.sectionIdentities.length; sectionIndex++){
                let sectionID = dataSource.sectionIdentities[sectionIndex];
                let sectionHeaderData = dataSource.getSectionHeaderData(sectionIndex);
                let sectionHeaderItem = {
                    sectionID:sectionID,
                    data:sectionHeaderData,
                };
                // dataList.add(sectionHeaderItem);

                let sectionRowIdentities = dataSource.rowIdentities[sectionIndex];
                for(let sectionRowIndex = 0; sectionRowIndex < sectionRowIdentities.length; sectionRowIndex++){
                    let rowID = sectionRowIdentities[sectionRowIndex];
                    let rowData = dataSource.getRowData(sectionIndex, sectionRowIndex);
                    let rowItem = {
                        sectionID:sectionID,
                        rowID:rowID,
                        data:rowData,
                    };
                    dataList.add(rowItem);
                }
            }
        }

        const recyclerViewStyle = [style, contentContainerStyle];

        //加载更多属性
        let loadMoreProps = {};
        if(enabledPullUp){  //底部刷新
            loadMoreProps.footerRefreshView = this._renderLoadMoreView();
            loadMoreProps.footerEndView = this._renderLoadMoreView(null, true);
            loadMoreProps.noInterceptTouchConfig = {y:'>1', isTop:true};
        }

        const recyclerView = (
            <View
                ref={'recyclerContainerView'}
                pointerEvents={'box-none'}
                style={[styles.container, recyclerViewStyle]}
                opacity={1}
            >
                <RecyclerView
                    ref={'recyclerView'}
                    {...otherProps}
                    scrollEventThrottle={scrollEventThrottle}
                    style={styles.list}
                    itemCount={dataList.length}
                    dataList={dataList}

                    onScroll={this._onScroll}
                    {...loadMoreProps}
                    footerRefreshViewHeight={MAX_REFRESH_HEIGHT}
                    onLoadMore={this._onLoadMore}
                    onRefreshState={this._onRefreshState}
                />
            </View>
        );

        // const recyclerView = (
        //     <View
        //         ref={'recyclerContainerView'}
        //         pointerEvents={'box-none'}
        //         style={[styles.container, recyclerViewStyle]}
        //         opacity={1}
        //     >
        //         <ListView
        //             ref={'recyclerView'}
        //             {...otherProps}
        //             dataSource={dataSource}
        //             scrollEventThrottle={scrollEventThrottle}
        //             style={styles.list}
        //             itemCount={dataList.length}
        //             dataList={dataList}
        //
        //             onScroll={this._onScroll}
        //             {...loadMoreProps}
        //             footerRefreshViewHeight={MAX_REFRESH_HEIGHT}
        //             onLoadMore={this._onLoadMore}
        //             onRefreshState={this._onRefreshState}
        //         />
        //     </View>
        // );

        if(enabledPullDown){    //下拉刷新
            return React.cloneElement(refreshControl, null, recyclerView);
        }

        return recyclerView;
    }
}

const styles = {
    container:{
        flex:1,
    },
    list:{
        flex:1,
    }
}